/*
 * DragonTalon.h
 *
 *  Created on: Feb 6, 2017
 *      Author: derekw
 */

#ifndef SRC_UTILS_DRAGONTALON_H_
#define SRC_UTILS_DRAGONTALON_H_

#include <CanTalon.h>

class DragonTalon : public CANTalon
{
	public:

		enum TALON_CONTROL_MODE
		{
			THROTTLE,
			POSITION,
			VELOCITY,
			VOLTAGE,
			FOLLOWER,
			MAGIC_MOTION,
			MAX_TALON_MODES
		};

		//------------------------------------------------------------------------------
		// Method:		<<constructor>>
		// Description:	Since we have methods on the CANTalon that don't appear to be
		// 				working, we have
		//------------------------------------------------------------------------------
		DragonTalon
		(
			const char*	deviceName,		// <I> - Name for motor
			int 		deviceID		// <I> - CAN ID
		);


		//------------------------------------------------------------------------------
		// Method:		Reset
		// Description:	Store the current encoder position, so GetEncoderPos will be
		//				relative to this value.
		// Returns:		void
		//------------------------------------------------------------------------------
		void Reset() override;


		void SetControlMode
		(
			TALON_CONTROL_MODE mode
		);

		void SetClosedLoopParams
		(
			double kP,
			double kI,
			double kD,
			double kF
		);

		//------------------------------------------------------------------------------
		// Method:		<<destructor>>
		// Description:	clean up
		//------------------------------------------------------------------------------
		virtual ~DragonTalon() = default;

		inline const char* GetName() { return m_name; }
		inline int GetID() { return GetDeviceID(); }


	private:
		const char*		m_name;
		int				m_zeroPos;						// Position when encoder was reset
		bool			m_sensorInverted;				// sensor is inverted
};



#endif /* SRC_SUBSYS_DRAGONTALON_H_ */
