/*
 * ChassisDefn.cpp
 */

#include <subsys/xmlParsing/ChassisDefn.h>
#include <subsys/xmlParsing/MotorDefn.h>

#include <subsys/components/DragonTalon.h>

#include <subsys/interfaces/IChassis.h>

#include <iostream>

#include <SmartDashboard/SmartDashboard.h>

#include <thirdParty/puxixml/pugixml.hpp>


//-----------------------------------------------------------------------
// Method:      ParseXML
// Description: Parse a Chassis XML element and create an IChassis from
//              its definition.
// Returns:     IChassis*        	chassis (or nullptr if XML
//                                  is ill-formed)
//-----------------------------------------------------------------------
IChassis* ChassisDefn::ParseXML
(
	pugi::xml_node      chassisNode
)
{
    IChassis* chassis = nullptr;
    enum CHASSIS_TYPE
	{
    	UNKNOWN_CHASSIS,
		TANK_CHASSIS,
		MECANUM_CHASSIS,
		MAX_CHASSIS_TYPES
	};
    CHASSIS_TYPE type 	= UNKNOWN_CHASSIS;
    float wheelBase 	= 0.0;
    float track 		= 0.0;
    bool hasError 		= false;

    // process attributes
    for (pugi::xml_attribute attr = chassisNode.first_attribute(); attr; attr = attr.next_attribute())
    {
    	printf( "Chassis attr: %s = %s \n ", attr.name(), attr.value() );

        if ( strcmp( attr.name(), "type" ) == 0 )
        {
        	if ( strcmp( attr.value(), "mecanum" ) == 0 )
        	{
        		type = MECANUM_CHASSIS;
        	}
        	else if ( strcmp( attr.value(), "tank") == 0 )
        	{
        		type = TANK_CHASSIS;
        	}
        	else
        	{
        		printf( "unknown chassis type %s \n ", attr.value() );
        		hasError = true;
        	}
        }
        else if ( strcmp( attr.name(), "wheelBase" ) == 0 )
        {
        	wheelBase = attr.as_float();
        }
        else if ( strcmp( attr.name(), "track" ) == 0 )
        {
        	track = attr.as_float();
        }
        else
        {
            std::cout << " invalid attribute " << attr.name();
            hasError = true;
        }
    }

    for (pugi::xml_node child = chassisNode.first_child(); child; child = child.next_sibling())
    {
    	printf( "Child %s \n ", child.name() );
    	if ( strcmp( child.name(), "motor") == 0 )
    	{
    		DragonTalon* talon = MotorDefn::ParseXML( child );
    	}
    	else if ( strcmp( child.name(), "analogSensor") == 0 )
    	{

    	}
    	else if ( strcmp( child.name(), "digitalSensor") == 0 )
    	{

    	}
    	else if ( strcmp( child.name(), "solenoid") == 0 )
    	{

    	}
    	else if ( strcmp( child.name(), "pcm") == 0 )
    	{

    	}
    }


    // create instance
    if ( !hasError )
    {
        printf( "Creating Chassis \n");

    }
    return chassis;
}

