/*
 * ProtoType.cpp
 *
 */
#include <iostream>


#include <SmartDashboard/SmartDashboard.h>

#include <thirdParty/puxixml/pugixml.hpp>


#include <test/ProtoType.h>

#include <subsys/components/DragonTalon.h>
#include <subsys/factories/ManipulatorFactory.h>
#include <subsys/ProtoSubsys.h>


ProtoType::ProtoType()
{
}

ProtoType::~ProtoType()
{
}


//-----------------------------------------------------------------------
// Method:      Init
// Description: Initializes the Prototype tuning.   This should
//              be called from the appropriate Init method in Robot.cpp
//
// Returns:     void
//-----------------------------------------------------------------------
void ProtoType::Init
(
	ITest::ITEST_CASE_ID   test    // <I> - test case identifier
)
{

    switch ( test )
    {
        case PROTOTYPE_TEST:
        	ParseXML();
            break;

        default:
        	printf( "ProtoType  invalid test identifier %d ", test );
            break;

    }

}



//-----------------------------------------------------------------------
// Method:      Periodic
// Description: This runs the prototype tuning periodic routine.
// Returns:     void
//-----------------------------------------------------------------------
void ProtoType::Periodic()
{
	ProtoSubsys* proto = ManipulatorFactory::GetPrototype();
	if ( proto != nullptr )
	{
		printf( "output to motors \n");
		proto->SetModes( m_talonModes );
		proto->SetTargets( m_talonTargets );
		proto->SetCoefficients( m_kPs, m_kIs, m_kDs, m_kFs );
	}
	/*
	for ( unsigned int inx=0; inx<m_talons.size(); ++inx )
	{
		DragonTalon* thisTalon = m_talons[inx];
	    if ( thisTalon != nullptr )
		{
	    	SmartDashboard::PutNumber( thisTalon->GetName(), thisTalon->GetID() );

	    	double value = thisTalon->Get();
	    	printf( "Motor %s %d %g \n", thisTalon->GetName(), thisTalon->GetID(), value );
	    	printf( "  Target %g \n", thisTalon->Get() );
	    	printf( "  P %g \n", thisTalon->GetP() );
	    	printf( "  I %g \n", thisTalon->GetI() );
	    	printf( "  D %g \n", thisTalon->GetD() );
	    	printf( "  F %g \n", thisTalon->GetF() );
	    	thisTalon->Set( m_talonTargets[inx] );
	    	printf( " speed %g \n", thisTalon->GetSpeed() );
	    	printf( " enc vel %d \n", thisTalon->GetEncVel() );
	    	printf( " position %g \n", thisTalon->GetPosition() );
	    	printf( " enc pos %d \n", thisTalon->GetEncPosition() );
		}
	}
		*/
}

//-----------------------------------------------------------------------------------
// Method:          Stop
// Description:     Stop the Prototype tuning
// Returns:         void
//-----------------------------------------------------------------------------------
void ProtoType::Stop()
{
	// NO-OP
}

void ProtoType::ParseXML()
{
	const char* filename = "/home/lvuser/config/closedLoopParams/prototype.xml";

    pugi::xml_document doc;
    pugi::xml_parse_result result = doc.load_file( filename );

    if ( result )
    {
        std::cout << "XML [" << filename << "] parsed without errors, attr value: [" << doc.child("prototype").attribute("attr").value() << "]\n\n";
        pugi::xml_node parent = doc.root();

        for (pugi::xml_node node = parent.first_child(); node; node = node.next_sibling())
        {
            std::cout << "Node:";
            std::cout << " " << node.name();

            for (pugi::xml_attribute attr = node.first_attribute(); attr; attr = attr.next_attribute())
            {
                std::cout << " " << attr.name() << "=" << attr.value();
            }

            std::cout << std::endl;

            for (pugi::xml_node child = node.first_child(); child; child = child.next_sibling())
            {
            	int canID = 0;
            	const char* motorName = "";

            	DragonTalon::TALON_CONTROL_MODE mode = DragonTalon::VELOCITY;
            	double target = 0.0;
            	double kp = 0.0;
            	double ki = 0.0;
            	double kd = 0.0;
            	double kf = 0.0;

            	bool hasError = false;

                std::cout << "Child:";
                std::cout << " " << child.name();

                for (pugi::xml_attribute attr = child.first_attribute(); attr; attr = attr.next_attribute())
                {

                    std::cout << " " << attr.name() << "=" << attr.value();


                	if ( strcmp( attr.name(), "name" ) == 0 )
                	{
                		motorName = attr.value();
                	}
                	else if ( strcmp( attr.name(), "id" ) == 0 )
					{
						canID = attr.as_int();
					}
					else if ( strcmp( attr.name(), "target" ) == 0 )
					{
						target = attr.as_double();
					}
					else if ( strcmp( attr.name(), "proportional" ) == 0 )
					{
						kp = attr.as_double();
					}
					else if ( strcmp( attr.name(), "integral" ) == 0 )
					{
						ki = attr.as_double();
					}
					else if ( strcmp( attr.name(), "derivative" ) == 0 )
					{
						kd = attr.as_double();
					}
					else if ( strcmp( attr.name(), "feedForward" ) == 0 )
					{
						kf = attr.as_double();
					}
					else if ( strcmp( attr.name(), "control" ) == 0 )
					{
						if ( strcmp( attr.value(), "velocity") == 0 )
						{
							mode = DragonTalon::VELOCITY;
						}
						else if ( strcmp( attr.value(), "throttle" ) == 0 )
						{
							mode = DragonTalon::THROTTLE;
						}
						else if ( strcmp( attr.value(), "position" ) == 0 )
						{
							mode = DragonTalon::POSITION;
						}
						else if ( strcmp( attr.value(), "voltage" ) == 0 )
						{
							mode = DragonTalon::VOLTAGE;
						}
						else if ( strcmp( attr.value(), "follower" ) == 0 )
						{
							mode = DragonTalon::FOLLOWER;
						}
						else if ( strcmp( attr.value(), "magic_motion" ) == 0 )
						{
							mode = DragonTalon::MAGIC_MOTION;
						}
						else
						{
							std::cout << " invalid mode " << attr.value();
							hasError = true;
						}
					}
					else
					{
						std::cout << " invalid attribute " << attr.name();
						hasError = true;
					}
                }

                std::cout << std::endl;

                std::cout << "Motor:  " << motorName << " ID " << canID << std::endl;
                std::cout << "Control Mode: " << mode << std::endl;
                std::cout << "   Target: " << target << std::endl;
                std::cout << "   P: " << kp << std::endl;
                std::cout << "   i: " << ki << std::endl;
                std::cout << "   d: " << kd << std::endl;
                std::cout << "   f: " << kf << std::endl;

                if ( !hasError )
                {
                    m_talonModes.emplace_back( mode );
                    m_talonTargets.emplace_back( target );
                    m_kPs.emplace_back( kp );
                    m_kIs.emplace_back( ki );
                    m_kDs.emplace_back( kd );
                    m_kFs.emplace_back( kf );
                }
            }


        }

    }
    else
    {
        std::cout << "XML [" << filename << "] parsed with errors, attr value: [" << doc.child("prototype").attribute("attr").value() << "]\n";
        std::cout << "Error description: " << result.description() << "\n";
        std::cout << "Error offset: " << result.offset << " (error at [..." << (filename + result.offset) << "]\n\n";
    }
}

