/*
 * DragonTalon.cpp
 *
 */

#include <subsys/components/DragonTalon.h>
#include <CANTalon.h>

//------------------------------------------------------------------------------
// Method:		<<constructor>>
// Description:	Since we have methods on the CANTalon that don't appear to be
// 				working, we have
//------------------------------------------------------------------------------
DragonTalon::DragonTalon
(
	DragonTalon::TALON_TYPE	deviceType,		// <I> - Usage of the motor
	int 					deviceID		// <I> - CAN ID
) : CANTalon( deviceID ),
	m_type( deviceType ),
	m_zeroPos( 0 ),
	m_sensorInverted( false )
{
	ClearStickyFaults();
}

//------------------------------------------------------------------------------
// Method:		Reset
// Description:	Store the current encoder position, so GetEncoderPos will be
//				relative to this value.
// Returns:		void
//------------------------------------------------------------------------------
void DragonTalon::Reset()
{
	SetEncPosition(0.0);
}

//Note: for position, it takes rotations as target.
//For velocity, it takes desired RPM.
//Counts per rotation is set in chassis in accordance with constant in chassis map
void DragonTalon::SetControlMode
(
	TALON_CONTROL_MODE mode
)
{
	switch(mode)
	{
		case THROTTLE:
			CANTalon::SetControlMode(CANSpeedController::kPercentVbus);
			break;

		case FOLLOWER:
			CANTalon::SetControlMode(CANSpeedController::kFollower);
			break;

		case VELOCITY:
			CANTalon::SetControlMode(CANSpeedController::kSpeed);
			break;

		case POSITION:
			CANTalon::SetControlMode(CANSpeedController::kPosition);
			break;

		case VOLTAGE:
			CANTalon::SetControlMode(CANSpeedController::kVoltage);
			break;

		default:
			CANTalon::SetControlMode(CANSpeedController::kPercentVbus);
			break;

	}

}

void DragonTalon::SetClosedLoopParams
(
	double kP,
	double kI,
	double kD,
	double kF
)
{
	SetAllowableClosedLoopErr( 0 ); // always servo

	SetF(kF); //set feed-forward gain
	SetP(kP); //set p-gain
	SetI(kI); //set i-gain
	SetD(kD); //set d-gain
}

