/*
 * DragonTalon.h
 *
 *  Created on: Feb 6, 2017
 *      Author: derekw
 */

#ifndef SRC_UTILS_DRAGONTALON_H_
#define SRC_UTILS_DRAGONTALON_H_

#include <string>

#include <CanTalon.h>

class DragonTalon : public CANTalon
{
	public:

		enum TALON_TYPE
		{
			UNKNOWN_TALON_TYPE = -1,
			MECANUM_FRONT_LEFT,
			MECANUM_FRONT_RIGHT,
			MECANUM_BACK_LEFT,
			MECANUM_BACK_RIGHT,
			TANK_LEFT_MASTER,
			TANK_LEFT_SLAVE1,
			TANK_LEFT_SLAVE2,
			TANK_RIGHT_MASTER,
			TANK_RIGHT_SLAVE1,
			TANK_RIGHT_SLAVE2,
			CLIMBER_MASTER,
			CLIMBER_SLAVE,
			SHOOTER_MASTER,
			SHOOTER_SLAVE,
			MAX_TALON_TYPES
		};
		enum TALON_CONTROL_MODE
		{
			THROTTLE,
			POSITION,
			VELOCITY,
			VOLTAGE,
			FOLLOWER,
			MAGIC_MOTION,
			MAX_TALON_MODES
		};

		//------------------------------------------------------------------------------
		// Method:		<<constructor>>
		// Description:	Since we have methods on the CANTalon that don't appear to be
		// 				working, we have
		//------------------------------------------------------------------------------
		DragonTalon
		(
			DragonTalon::TALON_TYPE	deviceType,		// <I> - Usage of the motor
			int 					deviceID		// <I> - CAN ID
		);


		//------------------------------------------------------------------------------
		// Method:		Reset
		// Description:	Store the current encoder position, so GetEncoderPos will be
		//				relative to this value.
		// Returns:		void
		//------------------------------------------------------------------------------
		void Reset() override;


		//------------------------------------------------------------------------------
		// Method:		SetControlMode
		// Description:	Switch the control mode for the talon
		//
		//				Note: Changing the mode changes the units for the target:
		//						position - rotations is the target.
		//						velocity - RPM is the target.
		//Counts per rotation is set in chassis in accordance with constant in chassis map
		// Returns:		void
		//------------------------------------------------------------------------------
		void SetControlMode
		(
			TALON_CONTROL_MODE mode		// <I> - talon control mode
		);


		//------------------------------------------------------------------------------
		// Method:		SetClosedLoopParams
		// Description:	Set the closed loop control parameters (PIDF coefficients)
		// Returns:		void
		//------------------------------------------------------------------------------
		void SetClosedLoopParams
		(
			double kP,					// <I> - proportional coefficient
			double kI,					// <I> - integral coefficient
			double kD,					// <I> - derivative coefficient
			double kF					// <I> - feed-forward coefficient
		);

		//------------------------------------------------------------------------------
		// Method:		<<destructor>>
		// Description:	clean up
		//------------------------------------------------------------------------------
		virtual ~DragonTalon() = default;

		//------------------------------------------------------------------------------
		// Method:		GetName
		// Description:	Return the talon's name
		// Returns:		const char*		talon's name
		//------------------------------------------------------------------------------
//		inline const char* GetName() { return m_name.c_str(); }
//		inline const char* GetName() { return m_name; }

		//------------------------------------------------------------------------------
		// Method:		GetType
		// Description:	Return the talon's usatge type
		// Returns:		DragonTalon::TALON_TYPE			talon's type
		//------------------------------------------------------------------------------
		inline DragonTalon::TALON_TYPE GetType() { return m_type(); }

		//------------------------------------------------------------------------------
		// Method:		GetCANID
		// Description:	Return the talon's CAN ID
		// Returns:		int			talon's CAN ID
		//------------------------------------------------------------------------------
		inline int GetCANID() { return GetDeviceID(); }


	private:
//		std::string		m_name;							// Name for the talon
//		const char*		m_name;							// Name for the talon
		DragonTalon::TALON_TYPE	m_type;							// Talon usage type
		int						m_zeroPos;						// Position when encoder was reset
		bool					m_sensorInverted;				// sensor is inverted
};



#endif /* SRC_SUBSYS_DRAGONTALON_H_ */
