/*
 * TestDragonPad.cpp
 *
 */

#include <test/TestDragonXbox.h>
#include <teleopControl/DragonXbox.h>
#include <SmartDashboard/SmartDashboard.h>

using namespace frc;



//----------------------------------------------------------------------------------
// Method:      GetInstance
// Description: If there isn't an instance of desired this class, create it.
//              Otherwise, hand out the single instance.
// Returns:     TestDragonStick*    TestDragonStick singleton
//----------------------------------------------------------------------------------
TestDragonXbox* TestDragonXbox::m_instance = nullptr;//Init the instance to nullptr
TestDragonXbox* TestDragonXbox::GetInstance()
{
    if (TestDragonXbox::m_instance == nullptr)
    {
        TestDragonXbox::m_instance = new TestDragonXbox();
    }
    return TestDragonXbox::m_instance;
}

//-----------------------------------------------------------------------
 // Method:      <<constructor>>
 // Description: creates the test environment attributes
 //-----------------------------------------------------------------------
TestDragonXbox::TestDragonXbox() : m_xbox( new DragonXbox( 0 ) )
{

}

//-----------------------------------------------------------------------
// Method:      Init
// Description: Initializes the DragonXbox for the test.   This should
//              be called from the appropriate Init method in Robot.cpp
//
//              Note: all tests are described with their enum and have a
//              deadband eliminated.
// Returns:     void
//-----------------------------------------------------------------------
void TestDragonXbox::Init
(
	ITest::ITEST_CASE_ID   test    // <I> - test case identifier
)
{
    AXIS_SENSITIVITY_CURVE curve = AXIS_LINEAR;
    float                  scale = 1.0;

    switch ( test )
    {
        case DRAGONXBOX_LINEAR_FULL_SCALE:
            break;

        case DRAGONXBOX_LINEAR_HALF_SCALE:
            scale = 0.5;
            break;

        case DRAGONXBOX_CUBIC_FULL_SCALE:
            curve = AXIS_CUBIC;
            break;

        case DRAGONXBOX_CUBIC_HALF_SCALE:
            curve = AXIS_CUBIC;
            scale = 0.5;
            break;

        default:
        	printf( "TestDragonXBox  invalid test identifier %d ", test );
            break;

    }

    m_xbox->SetAxisProfile( XBOX_X_AXIS_LEFT_JOYSTICK, curve );
    m_xbox->SetAxisScale( XBOX_X_AXIS_LEFT_JOYSTICK, scale );

    m_xbox->SetAxisProfile( XBOX_Y_AXIS_LEFT_JOYSTICK, curve );
    m_xbox->SetAxisScale( XBOX_Y_AXIS_LEFT_JOYSTICK, scale );

    m_xbox->SetAxisProfile( XBOX_LEFT_TRIGGER, curve );
    m_xbox->SetAxisScale( XBOX_LEFT_TRIGGER, scale );

    m_xbox->SetAxisProfile( XBOX_RIGHT_TRIGGER, curve );
    m_xbox->SetAxisScale( XBOX_RIGHT_TRIGGER, scale );

    m_xbox->SetAxisProfile( XBOX_X_AXIS_RIGHT_JOYSTICK, curve );
    m_xbox->SetAxisScale( XBOX_X_AXIS_RIGHT_JOYSTICK, scale );

    m_xbox->SetAxisProfile( XBOX_Y_AXIS_RIGHT_JOYSTICK, curve );
    m_xbox->SetAxisScale( XBOX_Y_AXIS_RIGHT_JOYSTICK, scale );
}



//-----------------------------------------------------------------------
// Method:      Periodic
// Description: This displays the joystick axis values and button status
//              to the dashboard
// Returns:     void
//-----------------------------------------------------------------------
void TestDragonXbox::Periodic()
{

    SmartDashboard::PutNumber( "Left Joystick - X-Axis: ", m_xbox->GetAxisValue( XBOX_X_AXIS_LEFT_JOYSTICK ) );
    SmartDashboard::PutNumber( "Left Joystick - Y-Axis: ", m_xbox->GetAxisValue( XBOX_Y_AXIS_LEFT_JOYSTICK ) );

    SmartDashboard::PutNumber( "Left Joystick - Trigger: ", m_xbox->GetAxisValue( XBOX_LEFT_TRIGGER ) );


    SmartDashboard::PutNumber( "Right Joystick - X-Axis: ", m_xbox->GetAxisValue( XBOX_X_AXIS_RIGHT_JOYSTICK ) );
    SmartDashboard::PutNumber( "Right Joystick - Y-Axis: ", m_xbox->GetAxisValue( XBOX_Y_AXIS_RIGHT_JOYSTICK ) );

    SmartDashboard::PutNumber( "Right Joystick - Trigger: ", m_xbox->GetAxisValue( XBOX_RIGHT_TRIGGER ) );

    SmartDashboard::PutBoolean( "A Button: ", m_xbox->IsButtonPressed( XBOX_A_BUTTON ) );
    SmartDashboard::PutBoolean( "B Button: ", m_xbox->IsButtonPressed( XBOX_B_BUTTON ) );
    SmartDashboard::PutBoolean( "X Button: ", m_xbox->IsButtonPressed( XBOX_X_BUTTON ) );
    SmartDashboard::PutBoolean( "Y Button: ", m_xbox->IsButtonPressed( XBOX_Y_BUTTON ) );
    SmartDashboard::PutBoolean( "LEFT_BUMPER: ", m_xbox->IsButtonPressed( XBOX_LEFT_BUMPER ) );
    SmartDashboard::PutBoolean( "RIGHT_BUMPER: ", m_xbox->IsButtonPressed( XBOX_RIGHT_BUMPER ) );
    SmartDashboard::PutBoolean( "BACK_BUTTON: ", m_xbox->IsButtonPressed( XBOX_BACK_BUTTON ) );
    SmartDashboard::PutBoolean( "START_BUTTON: ", m_xbox->IsButtonPressed( XBOX_START_BUTTON ) );
    SmartDashboard::PutBoolean( "PRESS_LEFT_STICK: ", m_xbox->IsButtonPressed( XBOX_PRESS_LEFT_STICK ) );
    SmartDashboard::PutBoolean( "PRESS_RIGHT_STICK: ", m_xbox->IsButtonPressed( XBOX_PRESS_RIGHT_STICK ) );
    SmartDashboard::PutBoolean( "LEFT_TRIGGER_PRESSED: ", m_xbox->IsButtonPressed( XBOX_LEFT_TRIGGER_PRESSED ) );
    SmartDashboard::PutBoolean( "RIGHT_TRIGGER_PRESSED: ", m_xbox->IsButtonPressed( XBOX_RIGHT_TRIGGER_PRESSED ) );
}

//-----------------------------------------------------------------------------------
// Method:          Stop
// Description:     Stop the test
// Returns:         void
//-----------------------------------------------------------------------------------
void TestDragonXbox::Stop()
{
	// NO-OP
}


