/*
 * RobotMap.cpp
 *
 */

#include <CANTalon.h>
#include <subsys/RobotMap.h>


namespace Team302
{
	RobotMap* RobotMap::m_instance = nullptr;	//Init the instance to nullptr

	//----------------------------------------------------------------------------------
	// Method:      GetInstance
	// Description: If there isn't an instance of the RobotMap, create it.
	//              Otherwise, hand out the single instance.
	// Returns:     RobotMap*   RobotMap that contains the physical characteristics of
	//							the robot
	//----------------------------------------------------------------------------------
	RobotMap* RobotMap::GetInstance()
	{
		if (RobotMap::m_instance == nullptr)
		{
			RobotMap::m_instance = new RobotMap();
		}
		return RobotMap::m_instance;
	}

	// TODO:: Read XML File Definition
	//----------------------------------------------------------------------------------
	// Constructor:	RobotMap
	// Description: Initialize the object.
	//----------------------------------------------------------------------------------
	RobotMap::RobotMap()  : m_leftFrontDriveMotor( LEFT_FRONT_DRIVE_MOTOR ),
							m_leftBackDriveMotor( LEFT_BACK_DRIVE_MOTOR ),
							m_rightFrontDriveMotor( RIGHT_FRONT_DRIVE_MOTOR ),
							m_rightBackDriveMotor( RIGHT_BACK_DRIVE_MOTOR ),
							m_leftFrontDriveMotorInverted( IS_LEFT_FRONT_DRIVE_MOTOR_INVERTED ),
							m_leftBackDriveMotorInverted( IS_LEFT_BACK_DRIVE_MOTOR_INVERTED ),
							m_rightFrontDriveMotorInverted( IS_RIGHT_FRONT_DRIVE_MOTOR_INVERTED ),
							m_rightBackDriveMotorInverted( IS_RIGHT_BACK_DRIVE_MOTOR_INVERTED ),
							m_leftFrontDriveSensorInverted( IS_LEFT_FRONT_DRIVE_ENCODER_INVERTED ),
							m_leftBackDriveSensorInverted( IS_LEFT_BACK_DRIVE_ENCODER_INVERTED ),
							m_rightFrontDriveSensorInverted( IS_RIGHT_FRONT_DRIVE_ENCODER_INVERTED ),
							m_rightBackDriveSensorInverted( IS_RIGHT_BACK_DRIVE_ENCODER_INVERTED ),
							m_leftFrontDriveMotorFeedbackType( CANTalon::FeedbackDevice::QuadEncoder ),
							m_leftBackDriveMotorFeedbackType( CANTalon::FeedbackDevice::QuadEncoder ),
							m_rightFrontDriveMotorFeedbackType( CANTalon::FeedbackDevice::QuadEncoder ),
							m_rightBackDriveMotorFeedbackType( CANTalon::FeedbackDevice::QuadEncoder ),
							m_leftFrontCountsPerRev( 20 ),
							m_leftBackCountsPerRev( 20 ),
							m_rightFrontCountsPerRev( 20 ),
							m_rightBackCountsPerRev( 20 ),
							m_leftFrontTurnsPerRev( 0 ),
							m_leftBackTurnsPerRev( 0 ),
							m_rightFrontTurnsPerRev( 0 ),
							m_rightBackTurnsPerRev( 0 ),
							m_leftFrontGearRatio( 6.0 ),
							m_leftBackGearRatio( 6.0 ),
							m_rightFrontGearRatio( 6.0 ),
							m_rightBackGearRatio( 6.0 ),
							m_safetyEnabled( false ),
							m_leftFrontWheelDiameter( 4.0 ),
							m_leftBackWheelDiameter( 4.0 ),
							m_rightFrontWheelDiameter( 4.0 ),
							m_rightBackWheelDiameter( 4.0 ),
							m_leftFrontWheelCountToInchConversion( 1.0 ),
							m_leftBackWheelCountToInchConversion( 1.0 ),
							m_rightFrontWheelCountToInchConversion( 1.0 ),
							m_rightBackWheelCountToInchConversion( 1.0 )




	{
		m_leftFrontWheelCountToInchConversion  = 1.0 / m_leftFrontGearRatio;			// adjust counts for the gear ratio (how does the sensor see vs. the wheel itself)
		m_leftFrontWheelCountToInchConversion *=  ( m_leftFrontWheelDiameter * M_PI );  // inches per wheel revolution
		m_leftFrontWheelCountToInchConversion /= m_leftFrontCountsPerRev;				// adjust for counts per revolution (so now inches / count)

		m_leftBackWheelCountToInchConversion  = 1.0 / m_leftBackGearRatio;				// adjust counts for the gear ratio (how does the sensor see vs. the wheel itself)
		m_leftBackWheelCountToInchConversion *=  ( m_leftBackWheelDiameter * M_PI );  	// inches per wheel revolution
		m_leftBackWheelCountToInchConversion /= m_leftBackCountsPerRev;					// adjust for counts per revolution (so now inches / count)

		m_rightFrontWheelCountToInchConversion  = 1.0 / m_leftFrontGearRatio;			// adjust counts for the gear ratio (how does the sensor see vs. the wheel itself)
		m_rightFrontWheelCountToInchConversion *=  ( m_leftFrontWheelDiameter * M_PI ); // inches per wheel revolution
		m_rightFrontWheelCountToInchConversion /= m_leftFrontCountsPerRev;				// adjust for counts per revolution (so now inches / count)

		m_rightBackWheelCountToInchConversion  = 1.0 / m_rightBackGearRatio;			// adjust counts for the gear ratio (how does the sensor see vs. the wheel itself)
		m_rightBackWheelCountToInchConversion *=  ( m_rightBackWheelDiameter * M_PI );  // inches per wheel revolution
		m_rightBackWheelCountToInchConversion /= m_rightBackCountsPerRev;				// adjust for counts per revolution (so now inches / count)
	}

	//--------------------------------------------------------------------------------
	// 	Method:  		GetDriveMotorCANID
	//	Description:	Accessor for the Drive Motor CAN IDs
	//	Returns:		int		- CAN ID
	//--------------------------------------------------------------------------------
	int RobotMap::GetDriveMotorCANID
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		int motorID = -1;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				motorID = m_leftFrontDriveMotor;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				motorID = m_leftBackDriveMotorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				motorID = m_rightFrontDriveMotorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				motorID = m_rightBackDriveMotorInverted;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::GetDriveMotorCANID \n " );
				break;

		}

		return motorID;
	}

	//--------------------------------------------------------------------------------
	// 	Method:  		IsDriveMotorInverted
	//	Description:	Accessor for the whether the Drive Motor is inverted or not
	//	Returns:		bool		- true	= motor inverted
	//								  false = motor is not inverted
	//--------------------------------------------------------------------------------
	bool RobotMap::IsDriveMotorInverted
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		bool isInverted = false;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				isInverted = m_leftFrontDriveMotorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				isInverted = m_leftBackDriveMotorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				isInverted = m_rightFrontDriveMotorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				isInverted = m_rightBackDriveMotorInverted;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::IsDriveMotorInverted \n " );
				break;

		}
		return isInverted;
	}

	//--------------------------------------------------------------------------------
	// 	Method:  		GetDriveMotorFeedbackDevice
	//	Description:	Accessor for the Drive Motor feedback device
	//	Returns:		CANTalon::FeedbackDevice	feedback device type
	//--------------------------------------------------------------------------------
	CANTalon::FeedbackDevice RobotMap::GetDriveMotorFeedbackDevice
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		CANTalon::FeedbackDevice device = CANTalon::FeedbackDevice::QuadEncoder;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				device = m_leftFrontDriveMotorFeedbackType;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				device = m_leftBackDriveMotorFeedbackType;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				device = m_rightFrontDriveMotorFeedbackType;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				device = m_rightBackDriveMotorFeedbackType;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::GetDriveMotorFeedbackDevice \n " );
				break;

		}
		return device;
	}


	//--------------------------------------------------------------------------------
	// 	Method:  		IsSensorInverted
	//	Description:	Accessor for the whether the Drive Motor's sensor is inverted or not
	//	Returns:		bool		- true	= sensor is inverted
	//								  false = sensor is not inverted
	//--------------------------------------------------------------------------------
	bool RobotMap::IsSensorInverted
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		bool isInverted = false;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				isInverted = m_leftFrontDriveSensorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				isInverted = m_leftBackDriveSensorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				isInverted = m_rightFrontDriveSensorInverted;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				isInverted = m_rightBackDriveSensorInverted;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::IsDriveMotorInverted \n " );
				break;

		}
		return isInverted;
	}

	//--------------------------------------------------------------------------------
	// 	Method:  		GetCountsPerRev
	//	Description:	Accessor for the Drive Motor's encoder counts per revolution
	//	Returns:		uint16_t	number of counts per revolution
	//--------------------------------------------------------------------------------
	uint16_t RobotMap::GetCountsPerRev
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		uint16_t counts = 0;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				counts = m_leftFrontCountsPerRev;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				counts = m_leftBackCountsPerRev;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				counts = m_rightFrontCountsPerRev;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				counts = m_rightBackCountsPerRev;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::GetCountsPerRev \n " );
				break;

		}
		return counts;
	}

	//--------------------------------------------------------------------------------
	// 	Method:  		GetTurnsPerRev
	//	Description:	Accessor for the Drive Motor's analog sensor turns per revolution
	//	Returns:		uint16_t	number of turns per revolution
	//--------------------------------------------------------------------------------
	uint16_t RobotMap::GetTurnsPerRev
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		uint16_t counts = 0;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				counts = m_leftFrontTurnsPerRev;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				counts = m_leftBackTurnsPerRev;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				counts = m_rightFrontTurnsPerRev;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				counts = m_rightBackTurnsPerRev;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::GetTurnsPerRev \n " );
				break;

		}
		return counts;
	}

	//--------------------------------------------------------------------------------
	// 	Method:  		GetGearRatio
	//	Description:	Accessor for the Drive Motor's gear ratio
	//	Returns:		float	number of of the wheel shaft compared to the number of
	//							turns of the motor shaft if the sensor is mounted on the
	//							motor shaft.
	//--------------------------------------------------------------------------------
	float RobotMap::GetGearRatio
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		float ratio = 1.0;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				ratio = m_leftFrontGearRatio;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				ratio = m_leftBackGearRatio;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				ratio = m_rightFrontGearRatio;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				ratio = m_rightBackGearRatio;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::GetGearRatio \n " );
				break;

		}
		return ratio;
	}

	//--------------------------------------------------------------------------------
	// 	Method:  		GetWheelDiameter
	//	Description:	Accessor for the Drive Motor's wheel diameter
	//	Returns:		float		diameter in inches
	//--------------------------------------------------------------------------------
	float RobotMap::GetWheelDiameter
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{
		float diameter = 0.0;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				diameter = m_leftFrontWheelDiameter;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				diameter = m_leftBackWheelDiameter;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				diameter = m_rightFrontWheelDiameter;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				diameter = m_rightBackWheelDiameter;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::GetWheelDiameter \n " );
				break;

		}
		return diameter;
	}


	//--------------------------------------------------------------------------------
	// 	Method:  		GetCountsToInchConversionFactor
	//	Description:	Accessor for the Drive Motor's analog sensor turns per revolution
	//	Returns:		uint16_t	number of turns per revolution
	//--------------------------------------------------------------------------------
	float RobotMap::GetCountsToInchConversionFactor
	(
		RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
	) const
	{

		float factor = 1.0;
		switch ( motor )
		{
			case RobotMap::DRIVE_MOTORS::LEFT_FRONT:
				factor = m_leftFrontWheelCountToInchConversion;
				break;

			case RobotMap::DRIVE_MOTORS::LEFT_BACK:
				factor = m_leftBackWheelCountToInchConversion;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_FRONT:
				factor = m_rightFrontWheelCountToInchConversion;
				break;

			case RobotMap::DRIVE_MOTORS::RIGHT_BACK:
				factor = m_rightBackWheelCountToInchConversion;
				break;


			default:
				printf( "==>> ERROR: Invalid Drive Motor Requested in RobotMap::GetTurnsPerRev \n " );
				break;

		}
		return factor;
	}


}

