/*
 * RobotParser.cpp
 *
 *  Created on: Aug 18, 2017
 *      Author: witcpale
 */

#include <definitions/ChassisDefn.h>
#include <definitions/ManipulatorDefn.h>
#include <definitions/PCMDefn.h>
#include <thirdparty/pugixml/pugixml.hpp>

#include <iostream>
#include <string.h>
#include <subsys/RobotParser.h>
#include <vector>


namespace Team302
{

	//===============================================================================================
	// Method:
	// Description:
	// Returns:
	//===============================================================================================
	RobotParser::RobotParser() : m_chassis( nullptr ),
								 m_manipulators(),
								 m_pcm( nullptr )
	{
		m_manipulators.assign( ManipulatorDefn::manipulatorType::MAX_MANIPULATOR_TYPES, nullptr );
		pugi::xml_document doc;
		if ( doc.load_file( "config/Robot.xml" ) )
		{
			pugi::xml_node mainNode = doc.child( "Robot" );		// Get the main Robot node

			// Cycle through the children of the Robot node
			for ( pugi::xml_node element = mainNode.first_child(); element; element = mainNode.next_sibling() )
			{
				auto elementType = element.name();
				if ( strcmp( elementType, "Chassis" ) == 0 )
				{
					ParseChassis( element );
				}
				else if ( strcmp( elementType, "Manipulator" ) == 0 )
				{
					ParseManipulator( element );
				}
				else if ( strcmp( elementType, "PCM" ) == 0 )
				{
					ParsePCM( element );
				}
				else
				{
					printf( "skipping unknown element %s ", elementType );
				}
			}
		}
		// TODO Auto-generated constructor stub

	}

	//===============================================================================================
	// Method:
	// Description:
	// Returns:
	//===============================================================================================
	void RobotParser::ParseChassis
	(
		pugi::xml_node thisElement			// <I> - The Chassis XML element
	)
	{
		// Initialize the variables that hold the attribute values to create the chassis definition
		ChassisDefn::chassisType type      = ChassisDefn::chassisType::UNKNOWN_CHASSIS;
		float					 wheelBase = 0.0;
		float					 track     = 0.0;

		// Parse the attributes and update the corresponding variables
        for (pugi::xml_attribute attr = thisElement.first_attribute(); attr; attr = attr.next_attribute())
        {
        	auto attrName  = attr.name();
        	auto attrValue = attr.value();

        	if ( strcmp( attrName, "Type" ) == 0 )
        	{
        		if ( strcmp( attrValue, "Tank" ) == 0 )
        		{
        			type = ChassisDefn::chassisType::TANK_CHASSIS;
        		}
        		else if ( strcmp( attrValue, "Mecanum" ) == 0 )
        		{
        			type = ChassisDefn::chassisType::MECANUM_CHASSIS;
        		}
        		else
        		{
    				printf( "unknown chassis type %s ", attrValue );
        		}
        	}
        	else if ( strcmp( attrName, "WheelBase" ) == 0 )
        	{
        		wheelBase = atof( attrValue );    //convert to float
        	}
        	else if ( strcmp( attrName, "Track" ) == 0 )
        	{
        		wheelBase = atof( attrValue );    //convert to float
        	}
        	else
        	{
				printf( "skipping unknown chassis attribute %s ", attrName );
        	}
        }
        m_chassis = new ChassisDefn( type, wheelBase, track );  // Create the Chassis Definition


		for ( pugi::xml_node subElement = thisElement.first_child(); subElement; subElement = thisElement.next_sibling() )
		{
			auto elementType = subElement.name();
			if ( strcmp( elementType, "Motor" ) == 0 )
			{
			}
			else if ( strcmp( elementType, "Sensor" ) == 0 )
			{

			}
			else if ( strcmp( elementType, "Solenoid" ) == 0 )
			{

			}
			else
			{
				printf( "skipping unknown chassis element %s ", elementType );
			}
		}
	}


	//===============================================================================================
	// Method:
	// Description:
	// Returns:
	//===============================================================================================
	void RobotParser::ParseManipulator
	(
		pugi::xml_node thisElement			// <I> - The Manipulator XML element
	)
	{
		// Initialize the variables that hold the attribute values to create the chassis definition
		ManipulatorDefn::manipulatorType type      = ManipulatorDefn::manipulatorType::UNKNOWN_MANIPULATOR;

		// Parse the attributes and update the corresponding variables
        for (pugi::xml_attribute attr = thisElement.first_attribute(); attr; attr = attr.next_attribute())
        {
        	auto attrName  = attr.name();
        	auto attrValue = attr.value();

        	if ( strcmp( attrName, "Type" ) == 0 )
        	{
        		if ( strcmp( attrValue, "FuelHandler" ) == 0 )
        		{
        			type = ManipulatorDefn::manipulatorType::FUEL_HANDLER;
        		}
        		else if ( strcmp( attrValue, "GearHandler" ) == 0 )
        		{
        			type = ManipulatorDefn::manipulatorType::GEAR_HANDLER;
        		}
        		else if ( strcmp( attrValue, "Climber" ) == 0 )
        		{
        			type = ManipulatorDefn::manipulatorType::CLIMBER;
        		}
        		else
        		{
    				printf( "unknown manipulator type %s ", attrValue );
        		}
        	}
        	else
        	{
				printf( "skipping unknown manipulator attribute %s ", attrName );
        	}
        }
        if ( type > ManipulatorDefn::manipulatorType::UNKNOWN_MANIPULATOR && type < ManipulatorDefn::manipulatorType::MAX_MANIPULATOR_TYPES )
        {
        	auto slot = (int) type;
        	if ( m_manipulators[ slot ] == nullptr )
        	{
        		m_manipulators[ slot ] = new ManipulatorDefn( type );
				for ( pugi::xml_node subElement = thisElement.first_child(); subElement; subElement = thisElement.next_sibling() )
				{
					auto elementType = subElement.name();
					if ( strcmp( elementType, "Motor" ) == 0 )
					{
					}
					else if ( strcmp( elementType, "Sensor" ) == 0 )
					{

					}
					else if ( strcmp( elementType, "Solenoid" ) == 0 )
					{

					}
					else
					{
						printf( "skipping unknown manipulator element %s ", elementType );
					}
				}
        	}
        	else
        	{
				printf( "duplicate manipulator type %d ", type );
        	}


        }
	}

	//===============================================================================================
	// Method:
	// Description:
	// Returns:
	//===============================================================================================
	void RobotParser::ParsePCM
	(
		pugi::xml_node thisElement			// <I> - The PCM XML element
	)
	{
		// Initialize the variables that hold the attribute values to create the chassis definition
		int	 canID = 0;

		// Parse the attributes and update the corresponding variables
        for (pugi::xml_attribute attr = thisElement.first_attribute(); attr; attr = attr.next_attribute())
        {
        	auto attrName  = attr.name();
        	auto attrValue = attr.value();

        	if ( strcmp( attrName, "ID" ) == 0 )
        	{
        		canID = atoi( attrValue );
        	}
        	else
        	{
				printf( "skipping unknown PCM attribute %s ", attrName );
        	}
        }
        m_pcm = new PCMDefn( canID );  // Create the PCM Definition
	}

	//===============================================================================================
	// Method:
	// Description:
	// Returns:
	//===============================================================================================
	void RobotParser::ParseSolenoid
	(
		pugi::xml_node thisElement			// <I> - The Solenoid XML element
	)
	{
		// Initialize the variables that hold the attribute values to create the chassis definition
		std::string	 identifier = "Solenoid";
		int	 pcmID = 0;

		// Parse the attributes and update the corresponding variables
        for (pugi::xml_attribute attr = thisElement.first_attribute(); attr; attr = attr.next_attribute())
        {
        	auto attrName  = attr.name();
        	auto attrValue = attr.value();

        	if ( strcmp( attrName, "Identifier" ) == 0 )
        	{
        		identifier.clear();
        		identifier.assign( attrValue );
        	}
        	else if ( strcmp( attrName, "PcmID" ) == 0 )
        	{
        		pcmID = atoi( attrValue );
        	}
        	else
        	{
				printf( "skipping unknown PCM attribute %s ", attrName );
        	}
        }
	}

	//===============================================================================================
	// Method:
	// Description:
	// Returns:
	//===============================================================================================
	RobotParser::~RobotParser()
	{
		// TODO Auto-generated destructor stub
	}

} /* namespace Team302 */
