/*
 * TankDrive.cpp
 *
 *  Created on: Jan 14, 2017
 *      Author: Tumtu
 */

#include <driverInputs/DragonXbox.h>
#include <subsys/ChassisFactory.h>
#include <subsys/IChassis.h>
#include <teleopDrive/TankDrive.h>
#include <teleopDrive/TeleDriveBase.h>

using namespace Team302;

//--------------------------------------------------------------------------------
// Method:			TankDrive
// Description: 	Default Constructor
// Returns: 		void
//--------------------------------------------------------------------------------
TankDrive::TankDrive()
{
	DragonXbox::GetDriver()->SetCurve( DragonXbox::X_AXIS_RIGHT_JOYSTICK,DragonXbox::AXIS_CUBIC );
	DragonXbox::GetDriver()->SetCurve( DragonXbox::Y_AXIS_LEFT_JOYSTICK, DragonXbox::AXIS_CUBIC );
	DragonXbox::GetDriver()->SetCurve( DragonXbox::LEFT_TRIGGER, DragonXbox::AXIS_CUBIC );
	DragonXbox::GetDriver()->SetCurve( DragonXbox::RIGHT_TRIGGER, DragonXbox::AXIS_CUBIC );
}

//--------------------------------------------------------------------------------
// Method:			DriveWithJoysticks
// Description: 	Reads the DragonXBox stick values and sends the appropriate
//					values to the drive wheels
// Returns: 		void
//--------------------------------------------------------------------------------
void TankDrive::DriveWithJoysticks() const
{
	// update the current limiting option
	UpdateCurrentLimiting();

	// Initialize the motor speeds
    float leftFrontSpeed  = 0.0;
    float leftBackSpeed   = 0.0;
    float rightFrontSpeed = 0.0;
    float rightBackSpeed  = 0.0;


    // Strafe Left
    if ( DragonXbox::GetDriver()->GetButton( DragonXbox::LEFT_TRIGGER_PRESSED ) )
    {
		float power = DragonXbox::GetDriver()->GetAxis( DragonXbox::LEFT_TRIGGER );
		leftFrontSpeed  = -1.0 * power;
		leftBackSpeed   = power;
		rightFrontSpeed = leftBackSpeed;
		rightBackSpeed  = leftFrontSpeed;
    }

    // Strafe Right
    else if ( DragonXbox::GetDriver()->GetButton( DragonXbox::RIGHT_TRIGGER_PRESSED ) )
    {
		float power = DragonXbox::GetDriver()->GetAxis( DragonXbox::RIGHT_TRIGGER );
		leftFrontSpeed  = power;
		leftBackSpeed   = -1.0 * power;
		rightFrontSpeed = leftBackSpeed;
		rightBackSpeed  = leftFrontSpeed;
   }

    // Standard Drive
    else
    {
    	// Standard Drive:  Read the joystick values
		leftFrontSpeed = DragonXbox::GetDriver()->GetAxis( DragonXbox::Y_AXIS_LEFT_JOYSTICK );
		leftBackSpeed   = leftFrontSpeed;
		rightFrontSpeed = DragonXbox::GetDriver()->GetAxis( DragonXbox::X_AXIS_RIGHT_JOYSTICK );
		rightBackSpeed  = rightFrontSpeed;
    }


    // Set the motors
    m_chassis->SetOpenLoopPower( leftFrontSpeed, leftBackSpeed, rightFrontSpeed, rightBackSpeed );

}


