/*
 * DragonTalon.h
 *
 *  Created on: Feb 6, 2017
 *      Author: derekw
 */

#ifndef SRC_UTILS_DRAGONTALON_H_
#define SRC_UTILS_DRAGONTALON_H_

#include <string>

#include <CanTalon.h>

class DragonTalon : public CANTalon
{
	public:

		enum TALON_CONTROL_MODE
		{
			THROTTLE,
			POSITION,
			VELOCITY,
			VOLTAGE,
			FOLLOWER,
			MAGIC_MOTION,
			MAX_TALON_MODES
		};

		//------------------------------------------------------------------------------
		// Method:		<<constructor>>
		// Description:	Since we have methods on the CANTalon that don't appear to be
		// 				working, we have
		//------------------------------------------------------------------------------
		DragonTalon
		(
			const char*	deviceName,		// <I> - Name for motor
			int 		deviceID		// <I> - CAN ID
		);


		//------------------------------------------------------------------------------
		// Method:		Reset
		// Description:	Store the current encoder position, so GetEncoderPos will be
		//				relative to this value.
		// Returns:		void
		//------------------------------------------------------------------------------
		void Reset() override;


		//------------------------------------------------------------------------------
		// Method:		SetControlMode
		// Description:	Switch the control mode for the talon
		// Returns:		void
		//------------------------------------------------------------------------------
		void SetControlMode
		(
			TALON_CONTROL_MODE mode		// <I> - talon control mode
		);


		//------------------------------------------------------------------------------
		// Method:		SetClosedLoopParams
		// Description:	Set the closed loop control parameters (PIDF coefficients)
		// Returns:		void
		//------------------------------------------------------------------------------
		void SetClosedLoopParams
		(
			double kP,					// <I> - proportional coefficient
			double kI,					// <I> - integral coefficient
			double kD,					// <I> - derivative coefficient
			double kF					// <I> - feed-forward coefficient
		);

		//------------------------------------------------------------------------------
		// Method:		<<destructor>>
		// Description:	clean up
		//------------------------------------------------------------------------------
		virtual ~DragonTalon() = default;

		//------------------------------------------------------------------------------
		// Method:		GetName
		// Description:	Return the talon's name
		// Returns:		const char*		talon's name
		//------------------------------------------------------------------------------
//		inline const char* GetName() { return m_name.c_str(); }
		inline const char* GetName() { return m_name; }

		//------------------------------------------------------------------------------
		// Method:		GetCANID
		// Description:	Return the talon's CAN ID
		// Returns:		int			talon's CAN ID
		//------------------------------------------------------------------------------
		inline int GetCANID() { return GetDeviceID(); }


	private:
//		std::string		m_name;							// Name for the talon
		const char*		m_name;							// Name for the talon
		int				m_zeroPos;						// Position when encoder was reset
		bool			m_sensorInverted;				// sensor is inverted
};



#endif /* SRC_SUBSYS_DRAGONTALON_H_ */
