/*
 * TestTeleopControl.h
 *
 */

#ifndef SRC_TEST_TESTTELEOPCONTROL_H_
#define SRC_TEST_TESTTELEOPCONTROL_H_

#include <test/ITest.h>
#include <teleopControl/TeleopControl.h>

class TestTelopControl : public ITest
{
	public:

		//----------------------------------------------------------------------------------
		// Method:      GetInstance
		// Description: If there isn't an instance of desired this class, create it.
		//              Otherwise, hand out the single instance.
		// Returns:     TestDragonXbox*    TestDragonStick singleton
		//----------------------------------------------------------------------------------
		static TestTelopControl* GetInstance();

		//-----------------------------------------------------------------------
		// Method:      Init
		// Description: Initializes the OI for the test.   This should
		//              be called from the appropriate Init method in Robot.cpp
		//
		//              Note: all tests are described with their enum and have a
		//              deadband eliminated.
		// Returns:     void
		//-----------------------------------------------------------------------
		void Init
		(
			ITest::ITEST_CASE_ID   test    // <I> - test case identifier
		) override;

		//-----------------------------------------------------------------------
		// Method:      Periodic
		// Description: This displays the joystick axis values and button states
		//              to the dashboard
		// Returns:     void
		//-----------------------------------------------------------------------
		void Periodic() override;

        //-----------------------------------------------------------------------------------
        // Method:          Stop
        // Description:     Stop the test
        // Returns:         void
        //-----------------------------------------------------------------------------------
        void Stop() override;

	private:
		//-----------------------------------------------------------------------
		// Method:      <<constructor>>
		// Description: creates the test environment attributes
		//-----------------------------------------------------------------------
		TestTelopControl();


		//-----------------------------------------------------------------------
		// Method:      <<destructor>>
		// Description: cleans up other objects
		//-----------------------------------------------------------------------
		virtual ~TestTelopControl() = default;

		static TestTelopControl*                        m_instance; //make a singleton
		TeleopControl::FUNCTION_IDENTIFIER	 			m_test;


};


#endif /* SRC_TEST_TESTTELEOPCONTROL_H_ */
